@@ -16,6 +16,7 @@
 
 import inspect
 
+from ryu.controller import handler
 from ryu import ofproto
 from ryu import utils
 from . import event
@@ -39,8 +40,12 @@ def _ofp_msg_name_to_ev_name(msg_name):
 
 
 def ofp_msg_to_ev(msg):
-    name = _ofp_msg_name_to_ev_name(msg.__class__.__name__)
-    return _OFP_MSG_EVENTS[name](msg)
+    return ofp_msg_to_ev_cls(msg.__class__)(msg)
+
+
+def ofp_msg_to_ev_cls(msg_cls):
+    name = _ofp_msg_name_to_ev_name(msg_cls.__name__)
+    return _OFP_MSG_EVENTS[name]
 
 
 def _create_ofp_msg_ev_class(msg_cls):
@@ -57,7 +62,7 @@ def _create_ofp_msg_ev_class(msg_cls):
     _OFP_MSG_EVENTS[name] = cls
 
 
-def _create_ofp_msg_ev_from_module(ofp_praser):
+def _create_ofp_msg_ev_from_module(ofp_parser):
     # print mod
     for _k, cls in inspect.getmembers(ofp_parser, inspect.isclass):
         if not hasattr(cls, 'cls_msg_type'):
@@ -75,3 +80,6 @@ class EventOFPStateChange(event.EventBase):
     def __init__(self, dp):
         super(EventOFPStateChange, self).__init__()
         self.datapath = dp
+
+
+handler.register_service('ryu.controller.ofp_handler')
