@@ -167,8 +167,6 @@ class header(stringify.StringifyMixin):
     def __len__(self):
         pass
 
-# TODO: implement a class for routing header
-
 
 class opt_header(header):
     """an abstract class for Hop-by-Hop Options header and destination
@@ -330,6 +328,163 @@ class option(stringify.StringifyMixin):
         return self._MIN_LEN + self.len_
 
 
+@ipv6.register_header_type(inet.IPPROTO_ROUTING)
+class routing(header):
+    """An IPv6 Routing Header decoder class.
+    This class has only the parser method.
+
+    IPv6 Routing Header types.
+
+    http://www.iana.org/assignments/ipv6-parameters/ipv6-parameters.xhtml
+
+    +-----------+----------------------------------+-------------------+
+    | Value     | Description                      | Reference         |
+    +===========+==================================+===================+
+    | 0         | Source Route (DEPRECATED)        | [[IPV6]][RFC5095] |
+    +-----------+----------------------------------+-------------------+
+    | 1         | Nimrod (DEPRECATED 2009-05-06)   |                   |
+    +-----------+----------------------------------+-------------------+
+    | 2         | Type 2 Routing Header            | [RFC6275]         |
+    +-----------+----------------------------------+-------------------+
+    | 3         | RPL Source Route Header          | [RFC6554]         |
+    +-----------+----------------------------------+-------------------+
+    | 4 - 252   | Unassigned                       |                   |
+    +-----------+----------------------------------+-------------------+
+    | 253       | RFC3692-style Experiment 1 [2]   | [RFC4727]         |
+    +-----------+----------------------------------+-------------------+
+    | 254       | RFC3692-style Experiment 2 [2]   | [RFC4727]         |
+    +-----------+----------------------------------+-------------------+
+    | 255       | Reserved                         |                   |
+    +-----------+----------------------------------+-------------------+
+    """
+
+    TYPE = inet.IPPROTO_ROUTING
+
+    _OFFSET_LEN = struct.calcsize('!2B')
+
+    # IPv6 Routing Header Type
+    ROUTING_TYPE_2 = 0x02
+    ROUTING_TYPE_3 = 0x03
+
+    @classmethod
+    def parser(cls, buf):
+        (type_, ) = struct.unpack_from('!B', buf, cls._OFFSET_LEN)
+        switch = {
+            # TODO: make parsers of type2.
+            cls.ROUTING_TYPE_2: None,
+            cls.ROUTING_TYPE_3: routing_type3
+        }
+        cls_ = switch.get(type_)
+        if cls_:
+            return cls_.parser(buf)
+        else:
+            return None
+
+
+class routing_type3(header):
+    """
+    An IPv6 Routing Header for Source Routes with the RPL (RFC 6554)
+    encoder/decoder class.
+
+    This is used with ryu.lib.packet.ipv6.ipv6.
+
+    An instance has the following attributes at least.
+    Most of them are same to the on-wire counterparts but in host byte order.
+    __init__ takes the corresponding args in this order.
+
+    .. tabularcolumns:: |l|L|
+
+    ============== =======================================
+    Attribute      Description
+    ============== =======================================
+    nxt            Next Header
+    size           The length of the Routing header,
+                   not include the first 8 octet.
+                   (0 means automatically-calculate when encoding)
+    type           Identifies the particular Routing header variant.
+    seg            Number of route segments remaining.
+    cmpi           Number of prefix octets from segments 1 through n-1.
+    cmpe           Number of prefix octets from segment n.
+    pad            Number of octets that are used for padding
+                   after Address[n] at the end of the SRH.
+    adrs           Vector of addresses, numbered 1 to n.
+    ============== =======================================
+    """
+
+    _PACK_STR = '!BBBBBB2x'
+    _MIN_LEN = struct.calcsize(_PACK_STR)
+
+    def __init__(self, nxt=inet.IPPROTO_TCP, size=0,
+                 type_=3, seg=0, cmpi=0, cmpe=0, adrs=None):
+        super(routing_type3, self).__init__(nxt)
+        self.size = size
+        self.type_ = type_
+        self.seg = seg
+        self.cmpi = cmpi
+        self.cmpe = cmpe
+        adrs = adrs or []
+        assert isinstance(adrs, list)
+        self.adrs = adrs
+        self._pad = (8 - ((len(self.adrs) - 1) * (16 - self.cmpi) +
+                    (16 - self.cmpe) % 8)) % 8
+
+    @classmethod
+    def _get_size(cls, size):
+        return (int(size) + 1) * 8
+
+    @classmethod
+    def parser(cls, buf):
+        (nxt, size, type_, seg, cmp_, pad) = struct.unpack_from(
+            cls._PACK_STR, buf)
+        data = cls._MIN_LEN
+        header_len = cls._get_size(size)
+        cmpi = int(cmp_ >> 4)
+        cmpe = int(cmp_ & 0xf)
+        pad = int(pad >> 4)
+        adrs = []
+        if size:
+            # Address[1..n-1] has size (16 - CmprI) octets
+            adrs_len_i = 16 - cmpi
+            # Address[n] has size (16 - CmprE) octets
+            adrs_len_e = 16 - cmpe
+            form_i = "%ds" % adrs_len_i
+            form_e = "%ds" % adrs_len_e
+            while data < (header_len - (adrs_len_e + pad)):
+                (adr, ) = struct.unpack_from(form_i, buf[data:])
+                adr = ('\x00' * cmpi) + adr
+                adrs.append(addrconv.ipv6.bin_to_text(adr))
+                data += adrs_len_i
+            (adr, ) = struct.unpack_from(form_e, buf[data:])
+            adr = ('\x00' * cmpe) + adr
+            adrs.append(addrconv.ipv6.bin_to_text(adr))
+        return cls(nxt, size, type_, seg, cmpi, cmpe, adrs)
+
+    def serialize(self):
+        if self.size == 0:
+            self.size = ((len(self.adrs) - 1) * (16 - self.cmpi) +
+                        (16 - self.cmpe) + self._pad) / 8
+        buf = struct.pack(self._PACK_STR, self.nxt, self.size,
+                          self.type_, self.seg, (self.cmpi << 4) | self.cmpe,
+                          self._pad << 4)
+        buf = bytearray(buf)
+        if self.size:
+            form_i = "%ds" % (16 - self.cmpi)
+            form_e = "%ds" % (16 - self.cmpe)
+            slice_i = slice(self.cmpi, 16)
+            slice_e = slice(self.cmpe, 16)
+            for adr in self.adrs[:-1]:
+                buf.extend(
+                    struct.pack(
+                        form_i, addrconv.ipv6.text_to_bin(adr)[slice_i]))
+            buf.extend(struct.pack(
+                form_e,
+                addrconv.ipv6.text_to_bin(self.adrs[-1])[slice_e]))
+        return buf
+
+    def __len__(self):
+        return routing_type3._get_size(self.size)
+
+
 @ipv6.register_header_type(inet.IPPROTO_FRAGMENT)
 class fragment(header):
     """IPv6 (RFC 2460) fragment header encoder/decoder class.
@@ -409,7 +564,7 @@ class auth(header):
     _PACK_STR = '!BB2xII'
     _MIN_LEN = struct.calcsize(_PACK_STR)
 
-    def __init__(self, nxt=inet.IPPROTO_TCP, size=3, spi=0, seq=0,
+    def __init__(self, nxt=inet.IPPROTO_TCP, size=2, spi=0, seq=0,
                  data='\x00\x00\x00\x00'):
         super(auth, self).__init__(nxt)
         assert data is not None
@@ -420,7 +575,7 @@ class auth(header):
 
     @classmethod
     def _get_size(cls, size):
-        return (int(size) - 1) * 8
+        return (int(size) + 2) * 4
 
     @classmethod
     def parser(cls, buf):
