@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import abc
+import six
 import struct
 
 from ryu.lib import addrconv
@@ -227,9 +228,8 @@ class sctp(packet_base.PacketBase):
 # Chunk Types
 #
 #=======================================================================
+@six.add_metaclass(abc.ABCMeta)
 class chunk(stringify.StringifyMixin):
-
-    __metaclass__ = abc.ABCMeta
     _PACK_STR = '!BBH'
     _MIN_LEN = struct.calcsize(_PACK_STR)
 
@@ -252,9 +252,8 @@ class chunk(stringify.StringifyMixin):
         return self.length
 
 
+@six.add_metaclass(abc.ABCMeta)
 class chunk_init_base(chunk):
-
-    __metaclass__ = abc.ABCMeta
     _PACK_STR = '!BBHIIHHI'
     _MIN_LEN = struct.calcsize(_PACK_STR)
     _class_prefixes = ['param_']
@@ -304,9 +303,8 @@ class chunk_init_base(chunk):
         return str(buf)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class chunk_heartbeat_base(chunk):
-
-    __metaclass__ = abc.ABCMeta
     _class_prefixes = ['param_']
 
     def __init__(self, flags=0, length=0, info=None):
@@ -338,10 +336,8 @@ class chunk_heartbeat_base(chunk):
         return str(buf)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class chunk_ack_base(chunk):
-
-    __metaclass__ = abc.ABCMeta
-
     def __init__(self, flags=0, length=0):
         super(chunk_ack_base, self).__init__(self.chunk_type(), length)
         self.flags = flags
@@ -360,9 +356,8 @@ class chunk_ack_base(chunk):
         return buf
 
 
+@six.add_metaclass(abc.ABCMeta)
 class chunk_ecn_base(chunk):
-
-    __metaclass__ = abc.ABCMeta
     _PACK_STR = '!BBHI'
     _MIN_LEN = struct.calcsize(_PACK_STR)
 
@@ -1176,9 +1171,8 @@ class chunk_shutdown_complete(chunk):
 # Cause Code
 #
 #=======================================================================
+@six.add_metaclass(abc.ABCMeta)
 class cause(stringify.StringifyMixin):
-
-    __metaclass__ = abc.ABCMeta
     _PACK_STR = '!HH'
     _MIN_LEN = struct.calcsize(_PACK_STR)
 
@@ -1210,10 +1204,8 @@ class cause(stringify.StringifyMixin):
         return length
 
 
+@six.add_metaclass(abc.ABCMeta)
 class cause_with_value(cause):
-
-    __metaclass__ = abc.ABCMeta
-
     def __init__(self, value=None, length=0):
         super(cause_with_value, self).__init__(length)
         self.value = value
@@ -1777,9 +1769,8 @@ class cause_protocol_violation(cause_with_value):
 # Chunk Parameter Types
 #
 #=======================================================================
+@six.add_metaclass(abc.ABCMeta)
 class param(stringify.StringifyMixin):
-
-    __metaclass__ = abc.ABCMeta
     _PACK_STR = '!HH'
     _MIN_LEN = struct.calcsize(_PACK_STR)
 
