@@ -21,6 +21,7 @@ VRRPManager creates/deletes VRRPRounter instances dynamically.
 """
 
 import abc
+import six
 
 from ryu.base import app_manager
 from ryu.controller import event
@@ -103,9 +104,8 @@ class VRRPParams(object):
         return (3.0 * self.master_adver_interval) + self.skew_time
 
 
+@six.add_metaclass(abc.ABCMeta)
 class VRRPState(object):
-    __metaclass__ = abc.ABCMeta
-
     def __init__(self, vrrp_router):
         super(VRRPState, self).__init__()
         self.vrrp_router = vrrp_router
