@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import binascii
 import inspect
 import json
 import logging
@@ -779,6 +780,36 @@ class OfTester(app_manager.RyuApp):
                 break
 
     def _compare_flow(self, stats1, stats2):
+
+        def __reasm_match(match):
+            """ reassemble match_fields. """
+            mask_lengths = {'vlan_vid': 12 + 1,
+                            'ipv6_exthdr': 9}
+            match_fields = list()
+            for key, united_value in match.iteritems():
+                if isinstance(united_value, tuple):
+                    (value, mask) = united_value
+                    # look up oxm_fields.TypeDescr to get mask length.
+                    for ofb in ofproto_v1_3.oxm_types:
+                        if ofb.name == key:
+                            # create all one bits mask
+                            mask_len = mask_lengths.get(
+                                key, ofb.type.size * 8)
+                            all_one_bits = 2 ** mask_len - 1
+                            # convert mask to integer
+                            mask_bytes = ofb.type.from_user(mask)
+                            oxm_mask = int(binascii.hexlify(mask_bytes), 16)
+                            # when mask is all one bits, remove mask
+                            if oxm_mask & all_one_bits == all_one_bits:
+                                united_value = value
+                            # when mask is all zero bits, remove field.
+                            elif oxm_mask & all_one_bits == 0:
+                                united_value = None
+                            break
+                if united_value is not None:
+                    match_fields.append((key, united_value))
+            return match_fields
+
         attr_list = ['cookie', 'priority', 'hard_timeout', 'idle_timeout',
                      'table_id', 'instructions', 'match']
         for attr in attr_list:
@@ -787,6 +818,9 @@ class OfTester(app_manager.RyuApp):
             if attr == 'instructions':
                 value1 = sorted(value1)
                 value2 = sorted(value2)
+            elif attr == 'match':
+                value1 = sorted(__reasm_match(value1))
+                value2 = sorted(__reasm_match(value2))
             if str(value1) != str(value2):
                 flow_stats = []
                 for attr in attr_list:
